#include "sysinfo.h"
#include <sstream>
#include <tchar.h>
#include <strsafe.h>
#include "iphlpapi.h"

#ifdef UNICODE
#undef UNICODE
#endif

using namespace std;

Sysinfo::Sysinfo(void)
{
}

Sysinfo::~Sysinfo(void)
{
}

std::string Sysinfo::GetSysinfo()
{
	//Time Values
	SYSTEMTIME st;
	GetLocalTime(&st);

	//Memory values
	MEMORYSTATUS mstat;
	GlobalMemoryStatus(&mstat);

	//OS VALUES
	char OSName[256];
	GetOSDisplayString(OSName);
	
	std::stringstream ss;
	
	//Display Date
	ss << "<Test> \r\nSystem Information Plugin Copyright (c) 2007-2008 by Mikhail Polenin (The Architect)";
	ss << "\r\n\r\nTime:\r\n\tLocal Date: " << GetDayOfWeek(st.wDayOfWeek) << ", ";
	ss << st.wDay << " " << GetMonth(st.wMonth) << ", " << st.wYear;

	//Display Time
	ss << "\r\n\tLocal Time : ";
	if (st.wHour < 10) ss << "0";
	ss << st.wHour << ":";
	if (st.wMinute < 10) ss << "0";
	ss << st.wMinute << ":";
	if (st.wSecond < 10) ss << "0";
	ss << st.wSecond;

	//Display Operating System
	ss << "\r\n\r\nOperating System: " << "\r\n\t" << OSName;

	//Display Physical Memory Values
	DWORD dwAPh = mstat.dwAvailPhys;
	DWORD dwTPh = mstat.dwTotalPhys;
	DWORD dwUPh = dwTPh - dwAPh;
	ss << "\r\n\r\nPhysical Memory: ";
	ss << "\r\n\tPhysical Memory Available: " << dwAPh << " B (" << dwAPh / 1024 / 1024 << " MB) (" << GetPercentage((float)dwAPh, (float)dwTPh) << " %)";
	ss << "\r\n\tPhysical Memory Used: " << dwUPh << " B (" << dwUPh / 1024 / 1024 << " MB) (" << GetPercentage((float)dwUPh, (float)dwTPh) << " %)";
	ss << "\r\n\tPhysical Memory Total: " << dwTPh << " B (" << dwTPh / 1024 / 1024 << " MB)";

	//Display Paged Memory Values
	DWORD dwAPF = mstat.dwAvailPageFile;
	DWORD dwTPF = mstat.dwTotalPageFile;
	DWORD dwUPF = dwTPF - dwAPF;
	ss << "\r\n\r\nPage File: ";
	ss << "\r\n\tPage File Available: " << dwAPF << " B (" << dwAPF / 1024 / 1024 << " MB) (" << GetPercentage((float)dwAPF, (float)dwTPF) << " %)";
	ss << "\r\n\tPage File Used: " << dwUPF << " B (" << dwUPF / 1024 / 1024 << " MB) (" << GetPercentage((float)dwUPF, (float)dwTPF) << " %)";
	ss << "\r\n\tPage File Total: " << dwTPF << " B (" << dwTPF / 1024 / 1024 << " MB)";

	//Display Virtual Memory Values
	DWORD dwAV = mstat.dwAvailVirtual;
	DWORD dwTV = mstat.dwTotalVirtual;
	DWORD dwUV = dwTV - dwAV;
	ss << "\r\n\r\nVirtual Memory: ";
	ss << "\r\n\tVirtual Memory Available: " << dwAV << " B (" << dwAV / 1024 / 1024 << " MB) (" << GetPercentage((float)dwAV, (float)dwTV) << " %)";
	ss << "\r\n\tVirtual Memory Used: " << dwUV << " B (" << dwUV / 1024 / 1024 << " MB) (" << GetPercentage((float)dwUV, (float)dwTV) << " %)";
	ss << "\r\n\tVirtual Memory Total: " << dwTV << " B (" << dwTV / 1024 / 1024 << " MB)";

	DWORD dwProcId = GetCurrentProcessId();
	HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, NULL, dwProcId);
	FILETIME ftCreation;
	FILETIME ftExit;
	FILETIME ftKernel;
	FILETIME ftUser;
	SYSTEMTIME stKernel;
	
	GetProcessTimes(hProcess, &ftCreation, &ftExit, &ftKernel, &ftUser);
	long long totalWorkTime = (ftKernel.dwHighDateTime + ftKernel.dwLowDateTime) + (ftUser.dwHighDateTime + ftUser.dwLowDateTime);
	FILETIME ftTW;
	ftTW.dwHighDateTime = ftKernel.dwHighDateTime + ftUser.dwHighDateTime;
	ftTW.dwLowDateTime = ftKernel.dwLowDateTime + ftUser.dwLowDateTime;
	FileTimeToSystemTime(&ftTW, &stKernel);
	//Process Values
	SYSTEM_INFO si;
	GetSystemInfo(&si);
	SYSTEMTIME syst;
	GetSystemTime(&syst);

	FILETIME lastaccess;
	GetFileTime(hProcess, NULL, &lastaccess, NULL);
	FILETIME currentTime;
	SystemTimeToFileTime(&syst, &currentTime);
	long long totalTime = (currentTime.dwHighDateTime + currentTime.dwLowDateTime) - (lastaccess.dwHighDateTime + lastaccess.dwLowDateTime);
	//float usage = GetPercentage((float)totalWorkTime, (float)totalTime) / si.dwNumberOfProcessors;

	DWORD ProcID = GetCurrentProcessId();
	ss << "\r\n\r\nProcessor Information: ";
	ss << "\r\n\tNumber of CPUs: " << si.dwNumberOfProcessors;
	ss << "\r\n\tProcess Architecture: " << si.dwProcessorType;
	ss << "\r\n\tProcess CPU Time: ";
	if(stKernel.wHour < 10) ss << "0";
	ss << stKernel.wHour << ":";
	if(stKernel.wMinute < 10) ss << "0";
	ss << stKernel.wMinute << ":";
	if(stKernel.wSecond < 10) ss << "0";
	ss << stKernel.wSecond;
	//ss << "\r\n\tProcessor CPU Usage: ";
	//ss << usage << " %";
	if(si.dwNumberOfProcessors > 1)
	{
		int i = 0;
		for(i = 0; i < si.dwNumberOfProcessors; i++)
		{
			ss << "\r\n" << GetCPUInfo(i);
		}
	}
	else 
	{
		ss << "\r\n" << GetCPUInfo(0); 
	}
	ss << GetNICInfo();


	//Send Endpipe
	ss << "|";
	return ss.str();
}

std::string Sysinfo::GetCPUInfo(int CPUNum)
{
	std::stringstream stream;
			stream << "\r\n\tProcessor " << CPUNum + 1 << " Information: ";
			std::stringstream buff;
			buff << "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\" << CPUNum;
			char value[1024] = {0};
			HKEY hKey;
			DWORD sizeofbuff = 1024; 
			DWORD dwType = REG_SZ;
			RegOpenKeyEx(HKEY_LOCAL_MACHINE, buff.str().c_str(), NULL, KEY_QUERY_VALUE, &hKey);
			RegQueryValueEx(hKey, "ProcessorNameString", NULL, &dwType, (LPBYTE)value, &sizeofbuff);
			stream << "\r\n\t\tName: " << value;
			value[0] = 0;
			RegQueryValueEx(hKey, "Identifier", NULL, &dwType, (LPBYTE)value, &sizeofbuff);
			stream << "\r\n\t\tType: " << value;
			value[0] = 0;
			RegQueryValueEx(hKey, "VendorIdentifier", NULL, &dwType, (LPBYTE)value, &sizeofbuff);
			stream << "\r\n\t\tVendor: " << value;
			BYTE dwVal[4];
			dwType = REG_DWORD;
			RegQueryValueEx(hKey, "~MHz", NULL, &dwType, (LPBYTE)dwVal, &sizeofbuff);
			int byte1 = dwVal[0];
			int byte2 = dwVal[1];
			int byte3 = dwVal[2];
			int byte4 = dwVal[3];
			DWORD dwSpeed = byte1 + (byte2 * 256) + (byte3 * 256) + (byte4 * 256);
			stream << "\r\n\t\tSpeed: " << dwSpeed << "MHz";
	return stream.str();
}

std::string Sysinfo::GetCPUInfo(int CPUNum, int TypeofInfo)
{
	std::stringstream stream;
			std::stringstream buff;
			buff << "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\" << CPUNum;
			char value[1024] = {0};
			HKEY hKey;
			DWORD sizeofbuff = 1024; 
			DWORD dwType = REG_SZ;
			RegOpenKeyEx(HKEY_LOCAL_MACHINE, buff.str().c_str(), NULL, KEY_QUERY_VALUE, &hKey);
			switch(TypeofInfo)
			{
			case 1:
				{
					RegQueryValueEx(hKey, "ProcessorNameString", NULL, &dwType, (LPBYTE)value, &sizeofbuff);
					stream << value;
				} 
				break;
			case 2:
				{
					RegQueryValueEx(hKey, "Identifier", NULL, &dwType, (LPBYTE)value, &sizeofbuff);
					stream << value;
				}
				break;
			case 3:
				{
					RegQueryValueEx(hKey, "VendorIdentifier", NULL, &dwType, (LPBYTE)value, &sizeofbuff);
					stream << value;
				}
				break;
			case 4:
				{
					BYTE dwVal[4];
					dwType = REG_DWORD;
					RegQueryValueEx(hKey, "~MHz", NULL, &dwType, (LPBYTE)dwVal, &sizeofbuff);
					int byte1 = dwVal[0];
					int byte2 = dwVal[1];
					int byte3 = dwVal[2];
					int byte4 = dwVal[3];
					DWORD dwSpeed = byte1 + (byte2 * 256) + (byte3 * 256) + (byte4 * 256);
					stream << "\r\n\t\tSpeed: " << dwSpeed << "MHz";
				}
				break;
			default:
				stream << "Invalid Value on Information Query Type";
			}
		return stream.str();
}

std::string Sysinfo::GetNICInfo()
{
	DWORD sizeReq = NULL;
	std::stringstream NICInfo;
	NICInfo << "\r\n\r\nNetwork Interface Information: ";
	PMIB_IFTABLE pInfo = NULL;
	GetIfTable(NULL, &sizeReq, FALSE);
	pInfo = (PMIB_IFTABLE) new BYTE [sizeReq];
	memset (pInfo, 0, sizeReq);
	DWORD sizeToUse = sizeReq;
	if(!GetIfTable( (PMIB_IFTABLE)pInfo, 
     &sizeToUse, FALSE) == NO_ERROR)
	{
		NICInfo << "\r\n\r\nNO VALID INFORMATION";
	}
	else
	{
		for( unsigned int index = 0; index < 
		 ((PMIB_IFTABLE)pInfo)->dwNumEntries; index ++ )
		{
			MIB_IFROW& NICSpec = ((PMIB_IFTABLE)pInfo)->table[index];
			DWORD TotalIn = NICSpec.dwInNUcastPkts + NICSpec.dwInOctets + NICSpec.dwInUcastPkts + NICSpec.dwInUnknownProtos;
			DWORD TotalOut = NICSpec.dwOutNUcastPkts + NICSpec.dwOutOctets + NICSpec.dwOutUcastPkts;
			if(TotalIn > 0)
			{
				NICInfo << "\r\n\r\n\tAdapter " << index << " Information: ";
				NICInfo << "\r\n\t\tDescription: " << NICSpec.bDescr;
				NICInfo << "\r\n\t\tSpeed: " << NICSpec.dwSpeed / 1000000 << " MBps";
				NICInfo << "\r\n\t\tIn: " << TotalIn << " Packets";
				NICInfo << "\r\n\t\tOut: " << TotalOut << " Packets";
			}
		}
	}
	return NICInfo.str();
}

float Sysinfo::GetPercentage(float lVal, float hVal)
{
	float pct;
	pct = (100/hVal)*lVal;
	return pct;
}

std::string Sysinfo::GetDayOfWeek(short n)
{
	switch(n)
	{
	case 0:
		return "Sunday";
		break;
	case 1:
		return "Monday";
		break;
	case 2:
		return "Tuesday";
		break;
	case 3:
		return "Wednesday";
		break;
	case 4:
		return "Thursday";
		break;
	case 5:
		return "Friday";
		break;
	case 6:
		return "Saturday";
		break;
	default:
		return "INVALID VALUE!";
	}
}

std::string Sysinfo::GetMonth(short n)
{
	switch(n)
	{
	case 1:
		return "January";
		break;
	case 2:
		return "February";
		break;
	case 3:
		return "March";
		break;
	case 4:
		return "April";
		break;
	case 5:
		return "May";
		break;
	case 6:
		return "June";
		break;
	case 7:
		return "July";
		break;
	case 8:
		return "August";
		break;
	case 9:
		return "September";
		break;
	case 10:
		return "October";
		break;
	case 11:
		return "November";
		break;
	case 12:
		return "December";
		break;
	default:
		return "INVALID VALUE!";
	}
}

typedef void (WINAPI *PGNSI)(LPSYSTEM_INFO);
typedef BOOL (WINAPI *PGPI)(DWORD, DWORD, DWORD, DWORD, PDWORD);

BOOL Sysinfo::GetOSDisplayString(LPSTR pszOS)
{
   OSVERSIONINFOEX osvi;
   SYSTEM_INFO si;
   PGNSI pGNSI;
   PGPI pGPI;
   BOOL bOsVersionInfoEx;
   DWORD dwType;

   ZeroMemory(&si, sizeof(SYSTEM_INFO));
   ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));

   osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

   if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi)) )
      return 1;

   // Call GetNativeSystemInfo if supported or GetSystemInfo otherwise.

   pGNSI = (PGNSI) GetProcAddress(
      GetModuleHandle(TEXT("kernel32.dll")), 
      "GetNativeSystemInfo");
   if(NULL != pGNSI)
      pGNSI(&si);
   else GetSystemInfo(&si);

   if ( VER_PLATFORM_WIN32_NT==osvi.dwPlatformId && 
        osvi.dwMajorVersion > 4 )
   {
      StringCchCopy(pszOS, 256, TEXT("Microsoft "));

      // Test for the specific product.

      if ( osvi.dwMajorVersion == 6 && osvi.dwMinorVersion == 0 )
      {
         if( osvi.wProductType == VER_NT_WORKSTATION )
             StringCchCat(pszOS, 256, TEXT("Windows Vista "));
         else StringCchCat(pszOS, 256, TEXT("Windows Server 2008 " ));

         pGPI = (PGPI) GetProcAddress(
            GetModuleHandle(TEXT("kernel32.dll")), 
            "GetProductInfo");

         pGPI( 6, 0, 0, 0, &dwType);

         switch( dwType )
         {
            case PRODUCT_ULTIMATE:
               StringCchCat(pszOS, 256, TEXT("Ultimate Edition" ));
               break;
            case PRODUCT_HOME_PREMIUM:
               StringCchCat(pszOS, 256, TEXT("Home Premium Edition" ));
               break;
            case PRODUCT_HOME_BASIC:
               StringCchCat(pszOS, 256, TEXT("Home Basic Edition" ));
               break;
            case PRODUCT_ENTERPRISE:
               StringCchCat(pszOS, 256, TEXT("Enterprise Edition" ));
               break;
            case PRODUCT_BUSINESS:
               StringCchCat(pszOS, 256, TEXT("Business Edition" ));
               break;
            case PRODUCT_STARTER:
               StringCchCat(pszOS, 256, TEXT("Starter Edition" ));
               break;
            case PRODUCT_CLUSTER_SERVER:
               StringCchCat(pszOS, 256, TEXT("Cluster Server Edition" ));
               break;
            case PRODUCT_DATACENTER_SERVER:
               StringCchCat(pszOS, 256, TEXT("Datacenter Edition" ));
               break;
            case PRODUCT_DATACENTER_SERVER_CORE:
               StringCchCat(pszOS, 256, TEXT("Datacenter Edition (core installation)" ));
               break;
            case PRODUCT_ENTERPRISE_SERVER:
               StringCchCat(pszOS, 256, TEXT("Enterprise Edition" ));
               break;
            case PRODUCT_ENTERPRISE_SERVER_CORE:
               StringCchCat(pszOS, 256, TEXT("Enterprise Edition (core installation)" ));
               break;
            case PRODUCT_ENTERPRISE_SERVER_IA64:
               StringCchCat(pszOS, 256, TEXT("Enterprise Edition for Itanium-based Systems" ));
               break;
            case PRODUCT_SMALLBUSINESS_SERVER:
               StringCchCat(pszOS, 256, TEXT("Small Business Server" ));
               break;
            case PRODUCT_SMALLBUSINESS_SERVER_PREMIUM:
               StringCchCat(pszOS, 256, TEXT("Small Business Server Premium Edition" ));
               break;
            case PRODUCT_STANDARD_SERVER:
               StringCchCat(pszOS, 256, TEXT("Standard Edition" ));
               break;
            case PRODUCT_STANDARD_SERVER_CORE:
               StringCchCat(pszOS, 256, TEXT("Standard Edition (core installation)" ));
               break;
            case PRODUCT_WEB_SERVER:
               StringCchCat(pszOS, 256, TEXT("Web Server Edition" ));
               break;
         }
         if ( si.wProcessorArchitecture==PROCESSOR_ARCHITECTURE_AMD64 )
            StringCchCat(pszOS, 256, TEXT( ", 64-bit" ));
         else if (si.wProcessorArchitecture==PROCESSOR_ARCHITECTURE_INTEL )
            StringCchCat(pszOS, 256, TEXT(", 32-bit"));
      }

      if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2 )
      {
         if( GetSystemMetrics(SM_SERVERR2) )
            StringCchCat(pszOS, 256, TEXT( "Windows Server 2003 R2, "));
         else if ( osvi.wSuiteMask==VER_SUITE_STORAGE_SERVER )
            StringCchCat(pszOS, 256, TEXT( "Windows Storage Server 2003"));
         else if( osvi.wProductType == VER_NT_WORKSTATION &&
                  si.wProcessorArchitecture==PROCESSOR_ARCHITECTURE_AMD64)
         {
            StringCchCat(pszOS, 256, TEXT( "Windows XP Professional x64 Edition"));
         }
         else StringCchCat(pszOS, 256, TEXT("Windows Server 2003, "));

         // Test for the server type.
         if ( osvi.wProductType != VER_NT_WORKSTATION )
         {
            if ( si.wProcessorArchitecture==PROCESSOR_ARCHITECTURE_IA64 )
            {
                if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
                   StringCchCat(pszOS, 256, TEXT( "Datacenter Edition for Itanium-based Systems" ));
                else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
                   StringCchCat(pszOS, 256, TEXT( "Enterprise Edition for Itanium-based Systems" ));
            }

            else if ( si.wProcessorArchitecture==PROCESSOR_ARCHITECTURE_AMD64 )
            {
                if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
                   StringCchCat(pszOS, 256, TEXT( "Datacenter x64 Edition" ));
                else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
                   StringCchCat(pszOS, 256, TEXT( "Enterprise x64 Edition" ));
                else StringCchCat(pszOS, 256, TEXT( "Standard x64 Edition" ));
            }

            else
            {
                if ( osvi.wSuiteMask & VER_SUITE_COMPUTE_SERVER )
                   StringCchCat(pszOS, 256, TEXT( "Compute Cluster Edition" ));
                else if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
                   StringCchCat(pszOS, 256, TEXT( "Datacenter Edition" ));
                else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
                   StringCchCat(pszOS, 256, TEXT( "Enterprise Edition" ));
                else if ( osvi.wSuiteMask & VER_SUITE_BLADE )
                   StringCchCat(pszOS, 256, TEXT( "Web Edition" ));
                else StringCchCat(pszOS, 256, TEXT( "Standard Edition" ));
            }
         }
      }

      if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1 )
      {
         StringCchCat(pszOS, 256, TEXT("Windows XP "));
         if( osvi.wSuiteMask & VER_SUITE_PERSONAL )
            StringCchCat(pszOS, 256, TEXT( "Home Edition" ));
         else StringCchCat(pszOS, 256, TEXT( "Professional" ));
      }

      if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 )
      {
         StringCchCat(pszOS, 256, TEXT("Windows 2000 "));

         if ( osvi.wProductType == VER_NT_WORKSTATION )
         {
            StringCchCat(pszOS, 256, TEXT( "Professional" ));
         }
         else 
         {
            if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
               StringCchCat(pszOS, 256, TEXT( "Datacenter Server" ));
            else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
               StringCchCat(pszOS, 256, TEXT( "Advanced Server" ));
            else StringCchCat(pszOS, 256, TEXT( "Server" ));
         }
      }

       // Include service pack (if any) and build number.

      if( _tcslen(osvi.szCSDVersion) > 0 )
      {
          StringCchCat(pszOS, 256, TEXT(" ") );
          StringCchCat(pszOS, 256, osvi.szCSDVersion);
      }

      TCHAR buf[80];

      StringCchPrintf( buf, 80, TEXT(" (build %d)"), osvi.dwBuildNumber);
      StringCchCat(pszOS, 256, buf);

      return TRUE; 
   }

   else
   {
      return FALSE;
   }
}